import unicodedata
from kesi.susia.kongke import thiah, tshiau_tuasiosia, SuSiaTshoNgoo
from kesi.butkian.kongiong import KHIN_SIANN_HU


def tsuanTL(bun):
    si_khinsiann = bun.startswith(KHIN_SIANN_HU)
    if si_khinsiann:
        bun = bun.replace(KHIN_SIANN_HU, '')
    try:
        siann, un, tiau, tuasiosia = thiah(bun)
    except SuSiaTshoNgoo:
        return bun
    tailo = kapTL(siann, un, tiau)
    kiatko = tshiau_tuasiosia(tuasiosia, tailo)
    if si_khinsiann:
        kiatko = '--{}'.format(kiatko)
    return kiatko


def kapTL(siann, un, tiau):
    tiau = tsuan_TL_tiau(tiau)
    un = tau_tiauhu(un, tiau)
    return unicodedata.normalize(
        'NFC', siann + un)


def tsuan_TL_tiau(tiau):
    if tiau == '\u0306':
        return '\u030b'
    return tiau


def tau_tiauhu(un, tiau):
    lomaji = ''
    if 'a' in un:
        lomaji = un.replace('a', 'a' + tiau)
    elif 'oo' in un:
        lomaji = un.replace('oo', 'o' + tiau + 'o')
    elif 'ee' in un:
        lomaji = un.replace('ee', 'e' + tiau + 'e')
    elif 'ere' in un:
        lomaji = un.replace('ere', 'ere' + tiau)
    elif 'e' in un:
        lomaji = un.replace('e', 'e' + tiau)
    elif 'o' in un:
        lomaji = un.replace('o', 'o' + tiau)
    elif 'ui' in un:
        lomaji = un.replace('i', 'i' + tiau)
    elif 'iu' in un:
        lomaji = un.replace('u', 'u' + tiau)
    elif 'iri' in un:
        lomaji = un.replace('iri', 'iri' + tiau)
    elif 'i' in un:
        lomaji = un.replace('i', 'i' + tiau)
    elif 'u' in un:
        lomaji = un.replace('u', 'u' + tiau)
    elif 'ng' in un:
        # ng, mng
        lomaji = un.replace('ng', 'n' + tiau + 'g')
    elif 'm' in un:
        lomaji = un.replace('m', 'm' + tiau)
    return lomaji



import re

from kesi.butkian.kongiong import 組字式符號, 聲調符號, 標點符號, 敢是拼音字元,\
    敢是注音符號, LIAN_JI_HU, si_lomaji, normalize_taibun
from kesi.butkian.su import Su
from kesi.butkian.ji import Ji


class Ku:
    """
    ku = Ku()
    su = Su()
    ji = Ji()

    for su in ku:
        for ji in su:
            for guan in ji.hanlo:
    """
    _切組物件分詞 = re.compile('(([^ ｜]*[^ ]｜[^ ][^ ｜]*) ?|[^ ]+)')
    _是空白 = re.compile(r'[^\S\n]+')
    _是分字符號 = re.compile('{}+'.format(LIAN_JI_HU))
    _是數字 = set('0123456789')
    _是多字元標點 = re.compile(r'(\.\.\.)|(……)|(──)')

    def __init__(self, hanlo=None, lomaji=None):
        if hanlo is not None:
            hanlo = normalize_taibun(hanlo)
        if lomaji is not None:
            lomaji = normalize_taibun(lomaji)

        # Ku(lomaji='Goa')
        if hanlo is None:
            hanlo = lomaji
            lomaji = None

        if hanlo is None:
            self._su = []
        elif lomaji is None:
            tngji, tngji_khinsiann, si_bokangsu = (
                self._hunsik_tngji_tngsu(hanlo)
            )
            bun, khinsiann = self._tngsu(tngji, tngji_khinsiann, si_bokangsu)
            self._su = self._bun_tsuan_sutin(bun, khinsiann)
        else:
            """ 以羅馬字ê斷字斷詞為主，漢羅文--ê無效 """
            tnghanlo, _ps, _ps = self._hunsik_tngji_tngsu(hanlo)
            tnglomaji, tngji_khinsiann, si_bokangsu = (
                self._hunsik_tngji_tngsu(lomaji)
            )

            if len(tnghanlo) != len(tnglomaji):
                raise TuiBeTse(
                    'Kù bô pênn tn̂g: '
                    'Hanlo tn̂g {} jī, m̄-koh lomaji tn̂g {} jī'
                    .format(len(tnghanlo), len(tnglomaji)))

            hanlo_tin, _ps = self._tngsu(
                tnghanlo, tngji_khinsiann, si_bokangsu)
            lomaji_tin, khinsiann = self._tngsu(
                tnglomaji, tngji_khinsiann, si_bokangsu)

            self._su = self._phe_tsuan_sutin(
                hanlo_tin, lomaji_tin, khinsiann)

    def __str__(self):
        return self.hanlo

    def __iter__(self):
        yield from self._su

    def __getitem__(self, kui):
        return self._su[kui]

    def __len__(self):
        return len(self._su)

    def _bun_tsuan_sutin(self, bun_tin, khinsiann_tin):
        sutin = []
        for tsitsu, khinsiann in zip(bun_tin, khinsiann_tin):
            su = Su()
            for ji, si_khinsiann in zip(tsitsu, khinsiann):
                su.thiam(
                    Ji(ji, si_khinsiann=si_khinsiann)
                )
            sutin.append(su)
        return sutin

    def _phe_tsuan_sutin(self, hanlo_tin, lomaji_tin, khinsiann_tin):
        sutin = []
        for suhanlo_tin, sulomaji_tin, khinsiann in zip(
                hanlo_tin, lomaji_tin, khinsiann_tin):
            su = Su()
            for jihanlo, jilomaji, si_khinsiann in zip(
                    suhanlo_tin, sulomaji_tin, khinsiann):
                su.thiam(
                    Ji(jihanlo, lomaji=jilomaji,
                        si_khinsiann=si_khinsiann)
                )
            sutin.append(su)
        return sutin

    @property
    def hanlo(self):
        """
          會 kā 文本標準化：
          保留羅馬字 ê 空白，tshun--ê 空白會刣掉
        """
        bun = []
        頂一詞上尾是羅馬字 = False
        for su in self:
            suhanlo = su.hanlo
            #
            #  判斷愛先添空白符無
            #    H, H -> 'HH'
            #    H, L -> 'HL'
            #    L, L -> 'L L'
            #
            if 頂一詞上尾是羅馬字 and si_lomaji(suhanlo[0]):
                bun.append(' ')
            bun.append(suhanlo)
            頂一詞上尾是羅馬字 = si_lomaji(suhanlo[-1])
        return ''.join(bun)

    @property
    def lomaji(self):
        """
          會 kā 文本標準化：
          保留羅馬字 ê 空白，tshun--ê 空白會刣掉
        """
        bun = []
        頂一詞上尾是羅馬字 = False
        for su in self:
            sulomaji = su.lomaji
            #
            #  判斷愛先添空白符無
            #    H, H -> 'HH'
            #    H, L -> 'HL'
            #    L, L -> 'L L'
            #
            if 頂一詞上尾是羅馬字 and si_lomaji(sulomaji[0]):
                bun.append(' ')
            bun.append(sulomaji)
            頂一詞上尾是羅馬字 = si_lomaji(sulomaji[-1])
        return ''.join(bun)

    @property
    def kiphanlo(self):
        """
          會 kā 文本標準化：
          保留羅馬字 ê 空白，tshun--ê 空白會刣掉
        """
        bun = []
        頂一詞上尾是羅馬字 = False
        for su in self:
            suhanlo = su.kiphanlo
            #
            #  判斷愛先添空白符無
            #    H, H -> 'HH'
            #    H, L -> 'HL'
            #    L, L -> 'L L'
            #
            if 頂一詞上尾是羅馬字 and si_lomaji(suhanlo[0]):
                bun.append(' ')
            bun.append(suhanlo)
            頂一詞上尾是羅馬字 = si_lomaji(suhanlo[-1])
        return ''.join(bun)

    def thianji(self):
        """
        =臺灣言語工具.拆文分析器.篩出字物件
        """
        for tsit_su in self:
            yield from tsit_su

    def thiam(self, su):
        self._su.append(su)

    def POJ(self):
        sin_ku = Ku()
        for su in self:
            sin_ku.thiam(su.POJ())
        return sin_ku

    def TL(self):
        sin_ku = Ku()
        for su in self:
            sin_ku.thiam(su.TL())
        return sin_ku

    def _tngsu(self, 字陣列, 輕聲陣列, 佮後一个字無仝一个詞):
        巢狀詞陣列 = []
        巢狀輕聲陣列 = []
        位置 = 0
        while 位置 < len(字陣列):
            範圍 = 位置
            while 範圍 < len(佮後一个字無仝一个詞) and not 佮後一个字無仝一个詞[範圍]:
                範圍 += 1
            範圍 += 1
            巢狀詞陣列.append(字陣列[位置:範圍])
            巢狀輕聲陣列.append(輕聲陣列[位置:範圍])
            位置 = 範圍
        return 巢狀詞陣列, 巢狀輕聲陣列

    def _hunsik_tngji_tngsu(self, 語句):
        狀態 = self._分析狀態()
        if self._是空白.fullmatch(語句):
            return 狀態.分析結果()
        頂一个字 = None
        頂一个是連字符 = False
        頂一个是空白 = False
        頂一个是輕聲符號 = False
        頂一个是注音符號 = False
        位置 = 0
        while 位置 < len(語句):
            字 = 語句[位置]
            是連字符 = False
            是空白 = False
            是輕聲符號 = False
            是注音符號 = 敢是注音符號(字)
            if 狀態.是組字模式():
                狀態.這馬字加一个字元(字)
                狀態.組字模型加一个字元(字)
                if 狀態.組字長度有夠矣未():
                    狀態.這馬字好矣清掉囥入去字陣列()
                    狀態.變一般模式()
            elif 狀態.是一般模式():
                揣分字 = self._是分字符號.match(語句[位置:])
                if 揣分字:
                    # Hyphen
                    狀態.這馬字好矣清掉囥入去字陣列()
                    分字長度 = len(揣分字.group(0))
                    if 分字長度 == 1:
                        # LIAN_JI_HU
                        if not 狀態.敢有分析資料矣() or 頂一个是空白:
                            狀態.字陣列直接加一字(LIAN_JI_HU)
                            狀態.頂一字佮這馬的字無仝詞()
                        else:
                            狀態.頂一字佮這馬的字仝詞()
                            是連字符 = True
                            狀態.有連字符()
                    elif 分字長度 == 2:
                        是輕聲符號 = True
                        狀態.這陣是輕聲詞()
                        if not 頂一个是空白:
                            # hó --lah -> ['hó', '--lah']
                            # hó--lah -> ['hó--lah']
                            狀態.頂一字佮這馬的字仝詞()
                    else:
                        for _ in range(分字長度):
                            狀態.字陣列直接加一字(LIAN_JI_HU)
                            狀態.頂一字佮這馬的字無仝詞()
                    位置 += 分字長度 - 1
                elif self._是空白.fullmatch(字):
                    狀態.這馬字好矣清掉囥入去字陣列()
                    狀態.頂一字佮這馬的字無仝詞()
                    if 頂一个是連字符:
                        狀態.字陣列直接加一字(LIAN_JI_HU)
                        狀態.頂一字佮這馬的字無仝詞()
                    if 頂一个是輕聲符號:
                        狀態.字陣列直接加一字(LIAN_JI_HU)
                        狀態.頂一字佮這馬的字無仝詞()
                        狀態.字陣列直接加一字(LIAN_JI_HU)
                        狀態.頂一字佮這馬的字無仝詞()
                    是空白 = True
                # 羅馬字接做伙
                elif 敢是拼音字元(字):
                    # 頭前是羅馬字抑是輕聲、外來語的數字
                    # 「N1N1」、「g0v」濫做伙名詞，「sui2sui2」愛變做兩个字，予粗胚處理。
                    if (
                        not 敢是拼音字元(頂一个字) and
                        頂一个字 not in self._是數字
                    ):
                        # 頭前愛清掉
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                    if 頂一个是輕聲符號:
                        狀態.這馬是輕聲字()

                    狀態.這馬字加一个字元(字)
                # 數字
                elif 字 in self._是數字:
                    if (
                        頂一个字 not in self._是數字 and
                        not 敢是拼音字元(頂一个字) and
                        not 頂一个是注音符號
                    ):
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                    狀態.這馬字加一个字元(字)
                # 音標後壁可能有聲調符號 uainnˊ
                elif 字 in 聲調符號 and 敢是拼音字元(頂一个字):
                    狀態.這馬字加一个字元(字)
                # 處理注音，輕聲、注音、空三个後壁會當接注音
                elif 是注音符號:
                    if (
                        頂一个字 not in 聲調符號 and
                        not 頂一个是注音符號
                    ):
                        狀態.這馬字好矣清掉囥入去字陣列()
                    狀態.這馬字加一个字元(字)
                # 注音後壁會當接聲調
                elif 字 in 聲調符號 and 頂一个是注音符號:
                    狀態.這馬字加一个字元(字)

                elif 字 in 標點符號:
                    揣著多字元標點 = self._是多字元標點.match(語句[位置:])
                    if 字 == '•' and 狀態.上尾敢是o結尾():
                        狀態.這馬字加一个字元(字)
                    elif 揣著多字元標點:
                        符號 = 揣著多字元標點.group(0)
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                        狀態.字陣列直接加一字(符號)
                        狀態.頂一字佮這馬的字無仝詞()
                        位置 += len(符號) - 1
                    else:
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                        狀態.字陣列直接加一字(字)
                        狀態.頂一字佮這馬的字無仝詞()
                else:
                    if 狀態.這馬字敢全部攏數字():
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                    elif 敢是拼音字元(頂一个字):
                        狀態.這馬字好矣清掉囥入去字陣列()
                        狀態.頂一字佮這馬的字無仝詞()
                    else:
                        狀態.這馬字好矣清掉囥入去字陣列()
                    if 頂一个是輕聲符號:
                        狀態.這馬是輕聲字()

                    狀態.這馬字加一个字元(字)

                    if 字 in 組字式符號:
                        狀態.變組字模式()
                    else:
                        狀態.這馬字好矣清掉囥入去字陣列()
            位置 += 1
            頂一个字 = 字
            頂一个是連字符 = 是連字符
            頂一个是空白 = 是空白
            頂一个是輕聲符號 = 是輕聲符號
            頂一个是注音符號 = 是注音符號
        if 狀態.這馬字敢閣有物件():
            if 狀態.是一般模式():
                狀態.這馬字好矣清掉囥入去字陣列()
            else:
                raise 解析錯誤('語句組字式無完整，語句＝{0}'.format(str(語句)))
        if 頂一个是連字符:
            狀態.字陣列直接加一字(LIAN_JI_HU)
            狀態.頂一字佮這馬的字無仝詞()
        if 頂一个是輕聲符號:
            狀態.字陣列直接加一字(LIAN_JI_HU)
            狀態.頂一字佮這馬的字無仝詞()
            狀態.字陣列直接加一字(LIAN_JI_HU)
            狀態.頂一字佮這馬的字無仝詞()
        return 狀態.分析結果()

    class _分析狀態:

        def __init__(self):
            self._字陣列 = []
            self._輕聲陣列 = []
            self._佮後一个字無仝一个詞 = []
            self.變一般模式()
            # 組字式抑是數羅會超過一个字元
            self._這馬字 = ''
            self._這馬是輕聲字 = False
            self._這陣是輕聲詞 = False
            self._這陣是輕聲詞_而且是輕聲詞ê一部份 = False

        def 分析結果(self):
            return self._字陣列, self._輕聲陣列, self._佮後一个字無仝一个詞

        def 敢有分析資料矣(self):
            return len(self._字陣列) > 0 or self.這馬字敢閣有物件()

        def 這馬字敢閣有物件(self):
            return self._這馬字 != ''

        def 這馬字敢全部攏數字(self):
            return self._這馬字.isdigit()

        def 變一般模式(self):
            self._模式 = '一般'
            self._組字長度 = 0

        def 變組字模式(self):
            self._模式 = '組字'
            self._組字長度 = -1

        def 是一般模式(self):
            return self._模式 == '一般'

        def 是組字模式(self):
            return self._模式 == '組字'

        def 組字模型加一个字元(self, 字):
            if 字 in 組字式符號:
                self._組字長度 -= 1
            else:
                self._組字長度 += 1

        def 組字長度有夠矣未(self):
            return self._組字長度 == 1

        def 這馬字加一个字元(self, 字):
            self._這馬字 += 字

        def 這馬是輕聲字(self):
            self._這馬是輕聲字 = True

        def 這陣是輕聲詞(self):
            self._這陣是輕聲詞 = True
            self._這陣是輕聲詞_而且是輕聲詞ê一部份 = True

        def 有連字符(self):
            if self._這陣是輕聲詞:
                self._這陣是輕聲詞_而且是輕聲詞ê一部份 = True

        def 字陣列直接加一字(self, 字):
            self._字陣列.append(字)
            self._輕聲陣列.append(False)
            self._佮後一个字無仝一个詞.append(None)

        def 這馬字好矣清掉囥入去字陣列(self):
            if self._這馬字 != '':
                if self._這陣是輕聲詞:
                    if not self._這陣是輕聲詞_而且是輕聲詞ê一部份:
                        self.頂一字佮這馬的字無仝詞()
                        self._這陣是輕聲詞 = False
                    self._這陣是輕聲詞_而且是輕聲詞ê一部份 = False
                self._字陣列.append(self._這馬字)
                self._輕聲陣列.append(self._這馬是輕聲字)
                self._佮後一个字無仝一个詞.append(None)
                self._這馬字 = ''
                self._這馬是輕聲字 = False

        def 頂一字佮這馬的字仝詞(self):
            try:
                self._佮後一个字無仝一个詞[-1] = False
            except IndexError:
                pass

        def 頂一字佮這馬的字無仝詞(self):
            try:
                if self._佮後一个字無仝一个詞[-1] is None:
                    self._佮後一个字無仝一个詞[-1] = True
            except IndexError:
                pass

        def 上尾敢是o結尾(self):
            for o in ['o', 'ó', 'ò', 'ô', 'ǒ', 'ō', 'o̍', 'ő']:
                if self._這馬字.endswith(o):
                    return True
            return False


class 解析錯誤(Exception):
    pass


class TuiBeTse(ValueError):
    pass



import unicodedata
import re

from kesi.butkian.kongiong import KHIN_SIANN_HU
from kesi.susia.kongke import thiah, tshiau_tuasiosia, SuSiaTshoNgoo


def tsuanPOJ(bun):
    si_khinsiann = bun.startswith(KHIN_SIANN_HU)
    if si_khinsiann:
        bun = bun.replace(KHIN_SIANN_HU, '')
    try:
        siann, un, tiau, tuasiosia = thiah(bun)
    except SuSiaTshoNgoo:
        return bun
    poj = kapPOJ(siann, un, tiau)
    kiatko = tshiau_tuasiosia(tuasiosia, poj)
    if si_khinsiann:
        kiatko = '--{}'.format(kiatko)
    return kiatko


def kapPOJ(siann, un, tiau):
    return unicodedata.normalize(
        'NFC',
        臺羅數字調轉白話字.轉白話字(siann, un, tiau)
    )


class 臺羅數字調轉白話字():

    @classmethod
    def 轉白話字(cls, 聲, 韻, 調):
        白話字聲 = cls.轉白話字聲(聲)
        白話字韻 = cls.轉白話字韻(韻)
        白話字調 = cls.轉白話字調(調)
        白話字傳統調韻 = cls.白話字韻標傳統調(白話字韻, 白話字調)
        return (
            白話字聲 +
            白話字傳統調韻
        )

    @classmethod
    def 轉白話字聲(cls, 聲):
        白話字聲 = None
        if 聲 == 'ts':
            白話字聲 = 'ch'
        elif 聲 == 'tsh':
            白話字聲 = 'chh'
        else:
            白話字聲 = 聲
        return 白話字聲

    @classmethod
    def 轉白話字韻(cls, un):
        un = (
            un
            .replace('nn', 'ⁿ')
            .replace('oo', 'o͘')
            .replace('ua', 'oa')
            .replace('ue', 'oe')
            .replace('ing', 'eng')
            .replace('ik', 'ek')
        )
        return un

    @classmethod
    def 轉白話字調(cls, tiau):
        # ă a̋
        return tiau.replace('\u030b', '\u0306')

    @classmethod
    def 白話字韻標傳統調(cls, 白話字韻無調, 調):
        該標調的字 = ''
        if 'o͘' in 白話字韻無調:
            該標調的字 = 'o͘'
        elif re.search('(iau)|(oai)', 白話字韻無調):
            # 三元音 攏標佇a面頂
            該標調的字 = 'a'
        elif re.search('[aeiou]{2}', 白話字韻無調):
            # 雙元音
            if 白話字韻無調[0] == 'i':
                該標調的字 = 白話字韻無調[1]
            elif 白話字韻無調[1] == 'i':
                該標調的字 = 白話字韻無調[0]
            elif len(白話字韻無調) == 2:
                # xx
                該標調的字 = 白話字韻無調[0]
            elif 白話字韻無調[-1] == 'ⁿ' and 白話字韻無調[-2:] != 'hⁿ':
                # xxⁿ
                該標調的字 = 白話字韻無調[0]
            else:
                # xxn, xxng, xxhⁿ
                該標調的字 = 白話字韻無調[1]
        elif re.search('[aeiou]', 白話字韻無調):
            # 單元音
            該標調的字 = 白話字韻無調[0]
        elif 'ng' in 白話字韻無調:
            # ng, mng
            該標調的字 = 'n'
        elif 'm' in 白話字韻無調:
            該標調的字 = 'm'
        結果 = cls.加上白話字調符(白話字韻無調, 該標調的字, 調)
        return 結果

    @classmethod
    def 加上白話字調符(cls, 白話字韻無調, 標調字母, 調):
        return 白話字韻無調.replace(標調字母, 標調字母 + 調, 1)



import unicodedata
import re

from kesi.susia.pio import KONGKE_SIANNBO, KONGKE_UNBO

SI_TSUAN_TUASIA = 'SI_TSUAN_TUASIA'
SI_TSUAN_SIOSIA = 'SI_TSUAN_SIOSIA'
SI_THAU_TUASIA = 'SI_THAU_TUASIA'


TIAUHO_TIAUHU_PIO = {
    '1': '',
    '2': '\u0301',
    '3': '\u0300',
    '4': '',
    '5': '\u0302',
    '6': '\u030c',
    '7': '\u0304',
    '8': '\u030d',
    '9': '\u0306'
}


def khuann_tuasiosia(bun):
    latin = bun.replace('ⁿ', '')
    if latin.isupper():
        return SI_TSUAN_TUASIA
    elif latin.islower():
        return SI_TSUAN_SIOSIA
    else:
        return SI_THAU_TUASIA


def tshiau_tuasiosia(tuasiosia, bun):
    if tuasiosia == SI_TSUAN_TUASIA:
        return bun.upper()
    elif tuasiosia == SI_TSUAN_SIOSIA:
        return bun.lower()
    else:
        return bun.capitalize()


def thiah(lomaji):
    siannun, tiau = theh_sianntiau(lomaji)

    siannun_n = thong_n(siannun)
    tuasiosia = khuann_tuasiosia(siannun_n)

    siannun_se = siannun_n.lower()
    kongke = tsuan_kongke(siannun_se)
    siann, un = thiah_siannun(kongke)
    return siann, un, tiau, tuasiosia


def theh_sianntiau(lomaji):
    nfd = unicodedata.normalize('NFD', lomaji)
    # Guân-té tō sòo-jī-tiāu
    if nfd[-1] in TIAUHO_TIAUHU_PIO:
        return nfd[:-1], TIAUHO_TIAUHU_PIO[nfd[-1]]
    # Thuân-thóng-tiāu
    pitui = re.search(
        '\u0301|\u0300|\u0302|\u030c|\u0304|\u030d|\u030b|\u0306', nfd)
    try:
        tiau = pitui.group(0)
    except AttributeError:
        tiau = ''  # 1 or 4
    siannun = nfd.replace(tiau, '')
    return siannun, tiau


def thong_n(siannun):
    phinnim = re.sub('([a-z])(N)(h?)', r'\1ⁿ\3', siannun)
    phinnim = phinnim.replace('ᴺ', 'ⁿ')
    return phinnim


def tsuan_kongke(siannun):
    kiatko = (
        siannun
        .replace('ch', 'ts')
        .replace('ou', 'oo')
        .replace('o͘', 'oo')
        .replace('ⁿ', 'nn')
        .replace('oa', 'ua')
        .replace('oe', 'ue')
        .replace('eng', 'ing')
        .replace('ek', 'ik')
        .replace('oonn', 'onn')
    )
    return kiatko


def thiah_siannun(無調號音標):
    for 所在 in range(len(無調號音標)):
        聲母 = 無調號音標[:所在]
        if 聲母.lower() in KONGKE_SIANNBO:
            韻母 = 無調號音標[所在:]
            if 韻母.lower() in KONGKE_UNBO:
                return 聲母, 韻母
    raise SuSiaTshoNgoo('Bô tsit-khuán im-tsiat: {}'.format(無調號音標))


class SuSiaTshoNgoo(ValueError):
    pass



from .butkian.ku import Ku
from .butkian.ku import TuiBeTse
from .butkian.kongiong import normalize_taibun
from .butkian.kongiong import 標點符號 as PIAUTIAM
from .susia.kongke import thiah, SuSiaTshoNgoo


def kam_haphuat(tsit_ji_lomaji):
    try:
        thiah(tsit_ji_lomaji)
    except SuSiaTshoNgoo:
        return False
    return True


__all__ = [
    'Ku', 'TuiBeTse', 'normalize_taibun',
    'kam_haphuat', PIAUTIAM,
]



# -*- coding: utf-8 -*-
from kesi.butkian.kongiong import LIAN_JI_HU, si_lomaji
from kesi.butkian.kongiong import KHIN_SIANN_HU


class Su:

    def __init__(self):
        self._ji = []

    def __iter__(self):
        yield from self._ji

    def __len__(self):
        return len(self._ji)

    @property
    def hanlo(self):
        """
        會 kā 文本標準化：
        判斷愛先添連字符無
          H, H -> 'HH'
          H, L -> 'HL'
          L, H -> 'LH'
          L, L -> 'L-L'
          L, --L -> 'L--L'
        """
        buntin = []
        ting_ji_si_lomaji = False
        su_u_khinsiann = False

        for ji in self:
            jihanlo = ji.hanlo
            if ji.si_khinsiann:
                " Mài thinn liân-jī-hû "
                su_u_khinsiann = True
            elif ting_ji_si_lomaji and si_lomaji(jihanlo[0]):
                " L, L -> 'L-L' "
                buntin.append(LIAN_JI_HU)
            elif su_u_khinsiann:
                " --H, H -> '--H-H' "
                buntin.append(LIAN_JI_HU)
            buntin.append(jihanlo)
            ting_ji_si_lomaji = si_lomaji(jihanlo[-1])
        return ''.join(buntin)

    @property
    def lomaji(self):
        """
        會 kā 文本標準化：
        判斷愛先添連字符無
          H, H -> 'HH'
          H, L -> 'HL'
          L, H -> 'LH'
          L, L -> 'L-L'
          L, --L -> 'L--L'
        """
        buntin = []
        ting_ji_si_lomaji = False
        su_u_khinsiann = False

        for ji in self:
            jilomaji = ji.lomaji
            if ji.si_khinsiann:
                " Mài thinn liân-jī-hû "
                su_u_khinsiann = True
            elif ting_ji_si_lomaji and si_lomaji(jilomaji[0]):
                " L, L -> 'L-L' "
                buntin.append(LIAN_JI_HU)
            elif su_u_khinsiann:
                " --H, H -> '--H-H' "
                buntin.append(LIAN_JI_HU)
            buntin.append(jilomaji)
            ting_ji_si_lomaji = si_lomaji(jilomaji[-1])
        return ''.join(buntin)

    @property
    def kiphanlo(self):
        """
        會 kā 文本標準化：
        判斷愛先添連字符無
          H, H -> 'HH'
          H, L -> 'HL'
          L, H -> 'LH'
          L, L -> 'L-L'
          L, --L -> 'L--L'
        """
        buntin = []
        ting_ji_si_lomaji = False

        for ji in self:
            jihanlo = ji.kiphanlo
            if ting_ji_si_lomaji and si_lomaji(jihanlo[0]):
                " L, L -> 'L-L' "
                if ji.si_khinsiann:
                    buntin.append(KHIN_SIANN_HU)
                else:
                    buntin.append(LIAN_JI_HU)
            buntin.append(jihanlo)
            ting_ji_si_lomaji = si_lomaji(jihanlo[-1])
        return ''.join(buntin)

    def thiam(self, ji):
        self._ji.append(ji)

    def POJ(self):
        sin_su = Su()
        for ji in self:
            sin_su.thiam(ji.POJ())
        return sin_su

    def TL(self):
        sin_su = Su()
        for ji in self:
            sin_su.thiam(ji.TL())
        return sin_su



from setuptools import setup
from panpun import PANPUN


setup(
    name='KeSi',
    description='台語文NLP家私',
    long_description='Kā台文轉做電腦通利用ê格式。臺灣言語工具翻新。',
    packages=['kesi', 'kesi.butkian', 'kesi.susia', ],
    version=PANPUN,
    author='Tshuà Bûn-lī',
    author_email='ithuan@ithuan.tw',
    url='https://ithuan.tw/',
    download_url='https://github.com/i3thuan5/KeSi',
    keywords=[
        'Parser', 'Alignment', 'Taigi', 'Hanji', 'Lomaji',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Linguistic',
    ],
)



# -*- coding: utf-8 -*-

TL_SIANNBO = {
    'p', 'ph', 'm', 'b',
    't', 'th', 'n', 'l',
    'k', 'kh', 'ng', 'g',
    'ts', 'tsh', 's', 'j',
    'h', '',
}
# 臺灣閩南語羅馬字拼音方案使用手冊 + 臺灣語語音入門 + 教育部辭典的字
# 歌仔戲：枝頭 ki1 thiou5， 土 thou。目前教羅共ou轉oo（因為台華辭典按呢處理）
TL_UNBO = {
    'a', 'ah', 'ap', 'at', 'ak', 'ann', 'annh',
    'am', 'an', 'ang',
    'e', 'eh', 'enn', 'ennh',
    'i', 'ih', 'ip', 'it', 'ik', 'inn', 'innh',
    'im', 'in', 'ing',
    'o', 'oh',
    'oo', 'ooh', 'op', 'ok', 'om', 'ong', 'onn', 'onnh',
    'u', 'uh', 'ut', 'un',
    'ai', 'aih', 'ainn', 'ainnh',
    'au', 'auh', 'aunn', 'aunnh',
    'ia', 'iah', 'iap', 'iat', 'iak', 'iam', 'ian', 'iang', 'iann', 'iannh',
    'io', 'ioh',
    'iok', 'iong', 'ionn',
    'iu', 'iuh', 'iut', 'iunn', 'iunnh',
    'ua', 'uah', 'uat', 'uak', 'uan', 'uann', 'uannh',
    'ue', 'ueh', 'uenn', 'uennh',
    'ui', 'uih', 'uinn', 'uinnh',
    'iau', 'iauh', 'iaunn', 'iaunnh',
    'uai', 'uaih', 'uainn', 'uainnh',
    'm', 'mh', 'ng', 'ngh',
    'ioo', 'iooh',  # 諾 0hioo 0hiooh, 詞目總檔.csv:khan35 jioo51
    ######
    'er', 'erh', 'erm', 'ere', 'ereh',  # 泉　鍋
    'ee', 'eeh', 'uee',  # 漳　家
    'eng',
    'ir', 'irh', 'irp', 'irt', 'irk', 'irm', 'irn', 'irng', 'irinn',
    'ie',  # 鹿港偏泉腔
    'or', 'orh', 'ior', 'iorh',  # 蚵
    'uang',  # 金門偏泉腔　　風　huang1
    'oi', 'oih',  # 詞彙方言差.csv:硩⿰落去
}

KONGKE_SIANNBO = TL_SIANNBO
KONGKE_UNBO = TL_UNBO



# -*- coding: utf-8 -*-
from kesi.susia.POJ import tsuanPOJ
from kesi.susia.TL import tsuanTL
from kesi.butkian.kongiong import KHIN_SIANN_HU
from kesi.butkian.kongiong import si_lomaji


class Ji:

    def __init__(self, hanlo, lomaji=None, si_khinsiann=False):
        if si_khinsiann:
            self.hanlo = '--{}'.format(hanlo)
        else:
            self.hanlo = hanlo

        if lomaji and si_khinsiann:
            self.lomaji = '--{}'.format(lomaji)
        elif lomaji:
            self.lomaji = lomaji
        else:
            self.lomaji = self.hanlo

    @property
    def kiphanlo(self):
        if self.si_khinsiann and not si_lomaji(self.hanlo[2]):
            return self.hanlo[2:]
        return self.hanlo

    def POJ(self):
        if self.si_khinsiann:
            hanlo = self.hanlo[2:]
            lomaji = self.lomaji[2:]
        else:
            hanlo = self.hanlo
            lomaji = self.lomaji
        return Ji(
            tsuanPOJ(hanlo), tsuanPOJ(lomaji),
            si_khinsiann=self.si_khinsiann
        )

    def TL(self):
        if self.si_khinsiann:
            hanlo = self.hanlo[2:]
            lomaji = self.lomaji[2:]
        else:
            hanlo = self.hanlo
            lomaji = self.lomaji
        return Ji(
            tsuanTL(hanlo), tsuanTL(lomaji),
            si_khinsiann=self.si_khinsiann
        )

    @property
    def si_khinsiann(self):
        return self.hanlo.startswith(KHIN_SIANN_HU)
